package org.orbroker

import JdbcCloser._
import org.orbroker.exception._
import org.orbroker.conv._
import org.orbroker.adapt.BrokerAdapter
import org.orbroker.callback.ExecutionCallback
import java.sql.{ SQLException, ResultSet, Connection, PreparedStatement }

private[orbroker] trait ModifyStatement extends SQLStatement with GenKeyProducer {

  def execute[K](token: Token[K], timeout: Int, conn: Connection, parms: Map[String, _], genKeyHandler: Option[K ⇒ Unit]): (Int, String, Seq[Any]) = {
    val parsed = statement(parms)
    val ps = parsed.prepareUpdate(conn, genKeyHandler.isDefined)
    try {
      ps.setQueryTimeout(timeout)
      val values = setParms(token, ps, parsed.parmDefs, parms)
      val rowsUpdated = ps.executeUpdate()
      genKeyHandler.foreach { handleGeneratedKeys(token, _, ps.getGeneratedKeys, rowsUpdated) }
      (rowsUpdated, parsed.sql, values)
    } finally {
      ps.checkAndClose(id)
    }
  }

  import scala.collection.mutable.ArrayBuffer

  def executeBatch[G](
    token: Token[G], timeout: Int, conn: Connection, batchParms: (String, Traversable[_]),
    parms: Map[String, _], genKeyHandler: Option[G ⇒ Unit]): (Int, String, Seq[Seq[Any]]) = {
    val (batchParmName, batchParmValues) = batchParms
    val parsed = statement(parms)
    val ps = parsed.prepareUpdate(conn, genKeyHandler.isDefined)
    val batchValues = new ArrayBuffer[Seq[Any]]
    try {
      ps.setQueryTimeout(timeout)
      batchParmValues foreach { batchParm ⇒
        val allParms = parms + (batchParmName -> batchParm)
        batchValues += setParms(token, ps, parsed.parmDefs, allParms)
        ps.addBatch()
      }
      if (batchValues.isEmpty) {
        (0, parsed.sql, batchValues)
      } else {
        val rowsUpdated = ps.executeBatch().view.filter(_ >= 0).sum
        genKeyHandler foreach { handleGeneratedKeys(token, _, ps.getGeneratedKeys, rowsUpdated) }
        (rowsUpdated, parsed.sql, batchValues)
      }
    } finally {
      ps.checkAndClose(id)
    }
  }

}
