package org.orbroker.config.dynamic

import org.orbroker._
import freemarker.template.Configuration

/**
 * FreeMarker support for [[BrokerBuilder]].
 * @author Nils Kilden-Pedersen
 */
trait FreeMarkerSupport extends DynamicSupport {

  /** Optional template engine properties. */
  var freemarkerProps: Map[String, String] = Map("number_format"->"0")

  private lazy val freeMarkerConfig: Configuration = {
    val config = new Configuration
    for ((key, value) ← freemarkerProps)
      config.setSetting(key, value)
    config
  }

  protected abstract override 
  def dynamicStatement(id: Symbol, sql: String, sqlLines: Seq[String]): Option[SQLStatement] = {
    val superDynamic = super.dynamicStatement(id, sql, sqlLines)
    if (superDynamic == None && SQLStatement.isFreeMarker(sql)) {
      Some(new FreeMarkerStatement(id, sqlLines, trimSQL, callback, adapter, freeMarkerConfig) with ModifyStatement with QueryStatement)
    } else {
      superDynamic
    }
  }

}